@@ -1,6 +1,6 @@
 
 
-''' 
+'''
 -----------------------------
     Start APP
 -----------------------------
@@ -13,6 +13,8 @@ import flask_mail
 from flask import __version__
 from werkzeug.utils import secure_filename
 from flask_sqlalchemy import SQLAlchemy
+from flask_migrate import Migrate, MigrateCommand
+from flask_script import Manager, Shell
 from flask import *
 import re
 import hashlib
@@ -34,7 +36,7 @@ except:
 # Global Var
 verify_registered_email = list()
 ALLOWED_EXTENSIONS = {'txt', 'pdf', 'png', 'jpg', 'jpeg', 'gif'}
-Release = 'v0.1.2.8 Release'
+Release = 'v0.1.3 Release'
 
 
 # Get Configs File
@@ -43,7 +45,8 @@ serverurl = serverconf['url']
 # Init Flask
 app = Flask(__name__, static_url_path='/static', static_folder='public')
 app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///sqlite3.db'
-app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = True
+app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False
+
 app.config['MAIL_SERVER'] = serverconf['stuoe_smtp_host']
 app.config['MAIL_PORT'] = int(serverconf['stuoe_smtp_port'])
 app.config['MAIL_USERNAME'] = serverconf['stuoe_smtp_email']
@@ -58,8 +61,18 @@ Viewrender = view
 
 
 # Init DatabaseTable , Email , function
+manager = Manager(app)
 db = SQLAlchemy(app)
 mail = flask_mail.Mail(app)
+migrate = Migrate(app, db)
+
+
+star_for = db.Table('star_for',
+                    db.Column('star_user', db.Integer, db.ForeignKey(
+                        'User.id'), primary_key=True),
+                    db.Column('stared_post', db.Integer,
+                              db.ForeignKey('Post.id'), primary_key=True)
+                    )
 
 
 class User(db.Model):
@@ -116,7 +129,10 @@ class Post(db.Model):
     pushingtime = db.Column(db.Integer)
     tags = db.Column(db.Integer, db.ForeignKey('Tags.id'))
     lock = db.Column(db.Boolean, server_default='False')
+    top = db.Column(db.Boolean, server_default='False')
     reply = db.relationship("Reply", backref="Post")
+    star_user_list = db.relationship(
+        'User', secondary=star_for, backref=db.backref('Post'))
 
 
 class Messages(db.Model):
@@ -125,7 +141,9 @@ class Messages(db.Model):
     subject = db.Column(db.String(50))
     body = db.Column(db.String(3000))
     PostTime = db.Column(db.Integer)
+    "Postman指的是接收者"
     Postman = db.Column(db.String(40), db.ForeignKey('User.id'))
+    avater = db.Column(db.String(40))
 
 
 class Tags(db.Model):
@@ -148,6 +166,7 @@ class Reply(db.Model):
 
 db.create_all()
 
+
 # Check whether two groups are created
 
 if Group.query.filter_by(Group_name='注册用户').first() is None:
@@ -163,16 +182,17 @@ if Group.query.filter_by(Group_name='管理员').first() is None:
         Highest_authority_group=True)
     db.session.add(AdminGourp)
     db.session.commit()
-if Tags.query.filter_by(name="新鲜事").first() is None:
-    goodnewsTags = Tags(name='新鲜事', lock=False, icon='message')
-    db.session.add(goodnewsTags)
-    db.session.flush()
-    db.session.commit()
-if Tags.query.filter_by(name="咕咚事").first() is None:
-    goodnewsTags = Tags(name='咕咚事', lock=False, icon='child_care')
-    db.session.add(goodnewsTags)
-    db.session.flush()
-    db.session.commit()
+if Tags.query.filter_by().first() == None:
+    if Tags.query.filter_by(name="新鲜事").first() is None:
+        goodnewsTags = Tags(name='新鲜事', lock=False, icon='message')
+        db.session.add(goodnewsTags)
+        db.session.flush()
+        db.session.commit()
+    if Tags.query.filter_by(name="咕咚事").first() is None:
+        goodnewsTags = Tags(name='咕咚事', lock=False, icon='child_care')
+        db.session.add(goodnewsTags)
+        db.session.flush()
+        db.session.commit()
 
 
 class replyObj():
@@ -229,7 +249,7 @@ def db_create_user(email, password, nickname, user_group):
         user_dirty=False,
         registertime=time.time(),
         MessageToMailbox=True,
-        avater='http://identicon.relucks.org/' + str(random.randint(200,999)) + '?size=120')
+        avater='http://identicon.relucks.org/' + str(random.randint(200, 999)) + '?size=120')
     db.session.add(new_user)
     db.session.flush()
     db.session.commit()
@@ -269,6 +289,22 @@ def get_session(type='nickname'):
         session.clear()
 
 
+def getPost_list(tags='', num=30):
+    returnPost_list = []
+    if tags == '':
+        for i in Post.query.filter_by().all()[:num]:
+            if i.top:
+                returnPost_list.insert(0, i)
+            else:
+                returnPost_list.append(i)
+    else:
+        for i in Post.query.filter_by(tags=tags).all()[:num]:
+            if i.top:
+                returnPost_list.insert(0, i)
+            else:
+                returnPost_list.append(i)
+    return returnPost_list
+
 # Install
 
 
@@ -328,7 +364,7 @@ def installing_step():
 def send_index():
 
     if get_session() == False:
-        return Viewrender.gethome(auth=False, tagslist=Tags.query.filter_by().all(), postlist=reversed(Post.query.filter_by().all()[:8]), get_avater=get_avater)
+        return Viewrender.gethome(auth=False, tagslist=Tags.query.filter_by().all(), postlist=getPost_list(), get_avater=get_avater)
     else:
         try:
             serverconf['open_email']
@@ -339,7 +375,7 @@ def send_index():
             get_session('obj').verify_email = True
             db.session.flush()
             db.session.commit()
-        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=reversed(Post.query.filter_by().all()[:8]), get_avater=get_avater)
+        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=getPost_list(), get_avater=get_avater)
 
 
 @app.route('/logout')
@@ -391,15 +427,32 @@ def post_pages(pid):
             replyList=replyList)
 
 
+@app.route('/stared/<pid>')
+def post_stared(pid):
+    obj = db_getpostByid(pid)
+    if obj is None:
+        return abort(404)
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    for i in obj.star_user_list:
+        if i.id == user.id:
+            return redirect('/p/' + str(pid))
+    obj.star_user_list.append(user)
+    db.session.flush()
+    db.session.commit()
+    return redirect('/p/' + str(pid))
+
+
 @app.route('/t/<tid>')
 def get_tags(tid):
     if Tags.query.filter_by(id=tid).first() == None:
         return abort(404)
     tagsname = Tags.query.filter_by(id=tid).first().name
     if get_session() == False:
-        return Viewrender.gethome(auth=False, tagslist=Tags.query.filter_by().all(), postlist=reversed(Post.query.filter_by(tags=tagsname).all()[:8]), get_avater=get_avater)
+        return Viewrender.gethome(auth=False, tagslist=Tags.query.filter_by().all(), postlist=getPost_list(tags=tid), get_avater=get_avater)
     else:
-        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=reversed(Post.query.filter_by(tags=tagsname).all()[:8]), get_avater=get_avater)
+        return Viewrender.gethome(auth=True, userObj=get_session('obj'), tagslist=Tags.query.filter_by().all(), postlist=getPost_list(tags=tid), get_avater=get_avater)
 
 
 @app.route('/relation')
@@ -643,6 +696,88 @@ def adminWait_tid(tid):
     return redirect('/admin/tags')
 
 
+@app.route('/rmpost/<pid>')
+def rmpost(pid):
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    if not user.user_group == '管理员':
+        return abort(403)
+    post = Post.query.filter_by(id=pid).first()
+    if post == None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
+    for i in Reply.query.filter_by(father=pid).all():
+        db.session.delete(i)
+    db.session.delete(post)
+    db.session.flush()
+    db.session.commit()
+    return redirect('/')
+
+
+@app.route('/lock/<pid>')
+def lock(pid):
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    if not user.user_group == '管理员':
+        return abort(403)
+    post = Post.query.filter_by(id=pid).first()
+    if post == None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
+    post.lock = True
+    db.session.flush()
+    db.session.commit()
+    return redirect('/p/' + str(pid))
+
+
+@app.route('/unlock/<pid>')
+def unlock(pid):
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    if not user.user_group == '管理员':
+        return abort(403)
+    post = Post.query.filter_by(id=pid).first()
+    if post == None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
+    post.lock = False
+    db.session.flush()
+    db.session.commit()
+    return redirect('/p/' + str(pid))
+
+
+@app.route('/top/<pid>')
+def top(pid):
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    if not user.user_group == '管理员':
+        return abort(403)
+    post = Post.query.filter_by(id=pid).first()
+    if post == None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
+    post.top = True
+    db.session.flush()
+    db.session.commit()
+    return redirect('/p/' + str(pid))
+
+
+@app.route('/untop/<pid>')
+def untop(pid):
+    user = get_session('obj')
+    if not user:
+        return abort(403)
+    if not user.user_group == '管理员':
+        return abort(403)
+    post = Post.query.filter_by(id=pid).first()
+    if post == None:
+        return Viewrender.getMSG(auth=True, userObj=user, msg='帖子不存在')
+    post.top = False
+    db.session.flush()
+    db.session.commit()
+    return redirect('/p/' + str(pid))
+
+
 class SearchObj():
     def __init__(self, avater, url, name):
         self.avater = avater
@@ -776,7 +911,8 @@ def pushing_post():
         body=request.form['body'],
         pushingtime=time.time(),
         tags=request.form['tags'],
-        lock=False)
+        lock=False,
+        top=False)
     db.session.add(newPost)
     db.session.flush()
     db.session.commit()
@@ -834,6 +970,19 @@ def get_avater(userId):
     return obj.avater
 
 
-@app.route('/robots.txt')
+def postNotice(userId, title, body, user):
+    newNotice = Messages(subject=title, body=body, PostTime=time.time(), Postman=userId,
+                         avater='http://identicon.relucks.org/' + str(random.randint(200, 999)) + '?size=120')
+    msg = flask_mail.Message(recipients=db_getuserByid(userId).email,
+                             html=body,
+                             subject='网站通知:' + title,)
+    threading._start_new_thread(send_mail, (msg,))
+    current_app.logger.info(
+        "向" + db_getuserByid(userId).email + "发送了一个通知 标题:" + title + " 内容:" + body)
+
+    return redirect('/settings/check')
+
+
+@ app.route('/robots.txt')
 def robotsTxt():
     return serverconf['robots.txt']
