@@ -26,6 +26,7 @@ def getTimer(timetime, simple=False):
         return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(int(timetime)))
 
 
+
 def getTemplates(
         body='',
         title='',
@@ -41,7 +42,8 @@ def getTemplates(
             title=title,
             userObj=userObj,
             webtitle=serverconf['stuoe_name'],
-            js=serverconf['js'])
+            js=serverconf['js'],
+            auth=auth)
     else:
         nav = jinja2.Template(
             open(
@@ -50,7 +52,9 @@ def getTemplates(
                 encoding="utf-8").read()).render(
             title=serverconf['stuoe_name'],
             webtitle=serverconf['stuoe_name'],
-            js=serverconf['js'])
+            js=serverconf['js'],
+            userObj=userObj,
+            auth=auth)
     if base2:
         return jinja2.Template(
             open(
@@ -64,7 +68,8 @@ def getTemplates(
             colorText=serverconf['colorText'],
             per='admin',
             webtitle=serverconf['stuoe_name'],
-            js=serverconf['js'])
+            js=serverconf['js'],
+            auth=auth)
     else:
         return jinja2.Template(
             open(
@@ -78,12 +83,13 @@ def getTemplates(
             colorText=serverconf['colorText'],
             per='admin',
             webtitle=serverconf['stuoe_name'],
-            js=serverconf['js'])
+            js=serverconf['js'],
+            auth=auth)
 
 
 def gethome(auth=True, userObj='', tagslist='', postlist='', get_avater=''):
     body = jinja2.Template(open('storage/templates/index.html',
-                                'r', encoding="utf-8").read()).render(webtitle=serverconf['stuoe_name'], des=serverconf['stuoe_des'], userObj=userObj, tagslist=tagslist, postlist=list(postlist), get_avater=get_avater)
+                                'r', encoding="utf-8").read()).render(webtitle=serverconf['stuoe_name'], des=serverconf['stuoe_des'], userObj=userObj, tagslist=tagslist, postlist=list(postlist), get_avater=get_avater,auth=auth)
     return getTemplates(auth=auth, title='', body=body, userObj=userObj, base2=True)
 
 
@@ -156,7 +162,10 @@ def getPost(
         tags=Tags,
         ReplyList=replyList,
         list=list,
-        getTimer=getTimer)
+        getTimer=getTimer,
+        len=len,
+        auth=auth,
+        userObj=userObj)
     return getTemplates(
         body=body,
         auth=auth,
